@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later
 
 Copyright (C) 2014  Vyacheslav Trushkin
-Copyright (C) 2020-2024  Boian Bonev
+Copyright (C) 2020-2025  Boian Bonev
 
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
 
@@ -118,7 +118,7 @@ inline char *read_cmdline(int pid,int isshort) {
 	return rv;
 }
 
-inline void pidgen_cb(pg_cb cb,void *hint1,void *hint2) {
+inline void pidgen_cb(pg_cb cb,struct xxxid_stats_arr *hint1,filter_callback hint2) {
 	DIR *pr;
 
 	if ((pr=opendir("/proc"))) {
